# Installation

The instructions below will help you to properly install the generated package to the lavalite project.

## Location

Extract the package contents to the folder 

`/packages/category/categories/`

## Composer

Add the below entries in the `composer.json` file's autoload section and run the command `composer dump-autoload` in terminal.

```json

...
     "autoload": {
         ...

        "classmap": [
            ...
            
            "packages/category/categories/database/seeds",
            
            ...
        ],
        "psr-4": {
            ...
            
            "Category\\Categories\\": "packages/category/categories/src",
            
            ...
        }
    },
...

```

## Config

Add the entries in service provider in `config/app.php`

```php

...
    'providers'       => [
        ...
        
        Category\Categories\Providers\CategoriesServiceProvider::class,
        
        ...
    ],

    ...

    'alias'             => [
        ...
        
        'Categories'  => Category\Categories\Facades\Categories::class,
        
        ...
    ]
...

```

## Migrate

After service provider is set run the commapnd to migrate and seed the database.


    php artisan migrate
    php artisan db:seed --class=Category\\CategoriesTableSeeder

## Publishing


**Publishing configuration**

    php artisan vendor:publish --provider="Category\Categories\Providers\CategoriesServiceProvider" --tag="config"

**Publishing language**

    php artisan vendor:publish --provider="Category\Categories\Providers\CategoriesServiceProvider" --tag="lang"

**Publishing views**

    php artisan vendor:publish --provider="Category\Categories\Providers\CategoriesServiceProvider" --tag="view"


## URLs and APIs


### Web Urls

**Admin**

    http://path-to-route-folder/admin/categories/{modulename}

**User**

    http://path-to-route-folder/user/categories/{modulename}

**Public**

    http://path-to-route-folder/categories


### API endpoints

**List**
 
    http://path-to-route-folder/api/user/categories/{modulename}
    METHOD: GET

**Create**

    http://path-to-route-folder/api/user/categories/{modulename}
    METHOD: POST

**Edit**

    http://path-to-route-folder/api/user/categories/{modulename}/{id}
    METHOD: PUT

**Delete**

    http://path-to-route-folder/api/user/categories/{modulename}/{id}
    METHOD: DELETE

**Public List**

    http://path-to-route-folder/api/categories/{modulename}
    METHOD: GET

**Public Single**

    http://path-to-route-folder/api/categories/{modulename}/{slug}
    METHOD: GET